import React, { PropsWithChildren } from 'react';
import { Autocomplete, AutocompleteRenderGroupParams, List, AutocompleteRenderInputParams } from '@mui/material';
import { NsTypography, NsButton } from '../';
import { NsAutocompleteProps, OptionsProps, ComponentBoxProps } from './types.d';
import { StyledTextField } from './NsAutocomplete.styled';

const NsAutocomplete = ({
    id = 'custom_autocomplete_dropdown',
    options = [],
    renderOptions,
    loading,
    noOptionsText,
    onSelect,
    onChange,
    clear,
    seeMore,
    placeholder,
    className,
    iconStart,
    iconEnd,
    inputValue,
    value,
}: PropsWithChildren<NsAutocompleteProps>) => {
    const OptionUI = renderOptions;

    const handleSelect = (event: React.SyntheticEvent<Element, Event>, value: string | OptionsProps | null) => {
        onSelect ? onSelect(value) : null;
    };

    const handleInputChange = (event: React.SyntheticEvent<Element, Event>, newInputValue: string) => {
        onChange ? onChange(newInputValue) : null;
    };

    const renderInput = (params: AutocompleteRenderInputParams) => (
        <StyledTextField
            {...params}
            placeholder={placeholder}
            InputProps={{
                ...params.InputProps,
                startAdornment: iconStart,
                endAdornment: iconEnd,
            }}
        />
    );

    // TODO: @vijayprathap need to check if we can remove object spread of ...props and ...params in renderInput
    const renderOption = (props: ComponentBoxProps, option: OptionsProps) => (
        <li {...props}>
            <OptionUI {...props} option={option} />
        </li>
    );

    const renderOptionLabel = (option: OptionsProps) => option.value;

    const renderGroupBy = (option: OptionsProps) => option.group as string;

    const renderGroupName = ({ key, group, children }: AutocompleteRenderGroupParams) => (
        <li key={key}>
            {group ? (
                <NsTypography variant="p-m" weight="bold" className="p-3">
                    {group}
                </NsTypography>
            ) : null}
            <ul className="p-0">{children}</ul>
        </li>
    );

    const renderClearButton = () => {
        const { show, onClear, text = 'Clear Search' } = clear || {};
        if (!show) {
            return null;
        }
        return (
            <li className="text-right p-3">
                <NsButton onClick={onClear} data-testid="autocomplete-clear-id">
                    <NsTypography variant={'p-xs'} weight="bold">
                        {text}
                    </NsTypography>
                </NsButton>
            </li>
        );
    };

    const renderSeeMoreButton = () => {
        const { show, onSeeMore, text = 'See More Results' } = seeMore || {};
        if (!show) {
            return null;
        }
        return (
            <li className="text-center p-3">
                <NsButton onClick={onSeeMore} data-testid="autocomplete-loadmore-id">
                    <NsTypography variant="p-xs" weight="bold">
                        {text}
                    </NsTypography>
                </NsButton>
            </li>
        );
    };

    const renderListBoxComponent = ({ children, ...otherProps }: ComponentBoxProps) => (
        <List {...otherProps}>
            {renderClearButton()}
            {children}
            {renderSeeMoreButton()}
        </List>
    );

    return (
        <Autocomplete
            id={id}
            disablePortal
            className={className}
            options={options}
            loading={loading}
            noOptionsText={noOptionsText}
            groupBy={renderGroupBy}
            getOptionLabel={renderOptionLabel}
            renderOption={renderOption}
            renderInput={renderInput}
            ListboxComponent={renderListBoxComponent}
            renderGroup={renderGroupName}
            onChange={handleSelect}
            onInputChange={handleInputChange}
            value={value}
            inputValue={inputValue}
            freeSolo
        />
    );
};

export default NsAutocomplete;
